.frame {
  border-top: 1px solid #cccccc;
  border-bottom: 1px solid #cccccc;
  background: #f9f9f9;
  padding: 0 1rem 2rem 1rem;
  margin: 0.5rem 1.5rem;
  min-height: 18rem;
}

.frame .content {
    padding: 3rem 1rem 1rem 1rem;
}

.frame .content h2 {
    text-align: left;
    font-size: 1rem;
    margin-left: -1rem;
}

.footer {
  height: 4rem;
}