<!DOCTYPE html>
<html lang="en"
      xml:lang="en"
      xmlns="http://www.w3.org/1999/xhtml"
      xmlns:epub="http://www.idpf.org/2007/ops">
<head>
  <title>Table of Contents</title>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
</head>
<body>
  <nav epub:type="toc">
    {{ toc.xhtml }}
  </nav>
</body>
</html>
