{%- extends "default/html/template.html" %}
{%- block extra_css %}
<link rel="stylesheet" href="{{ "/media/css/novel.css" | rewrite_path }}">
{%- endblock %}
{%- block body %}
  {{ body.html }}
{%- endblock %}
{%- block footer %}
  {% include "default/html/nav.html" %}
{%- endblock %}
