<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en"
      xmlns:epub="http://www.idpf.org/2007/ops"  lang="en">
  <head>
    <title>{{ title|default("Document Title") }}</title>
    {%- block css %}
    <link rel="stylesheet" href="{{ "/media/css/epub.css" | rewrite_path }}"/>
    {% endblock %}
    {%- block extra_css %}
    {%- endblock %}
  </head>
  <body>
    {%- block body %}
    {{ body.xhtml }}
    {%- endblock %}
  </body>
</html>