.container {
  width: 840px;
  padding: 1rem 4rem 1rem 5rem;
}

article {
  width: 97%;
}

label.menu-toggle {
  margin-left: -4.2rem;
}

h2 .label {
  color: inherit;
  font-variant: small-caps;
  display: block;
  margin: 0 0 4rem 0;
  border: 0;
}

ul.toc li.toc-level-2 { /* chapter */
  display: inline-block;
  min-width: 30%;
  padding: 0.25em 0 0.0em 2em;
}

p:first-of-type {
  text-indent: 0;
}

p {
  margin-bottom: 0;
  text-indent: 2em;
}

p:first-of-type::first-letter {
  font-size: 200%;
}